/* liblwevent/src/liblwevent/400_signalfd.c
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv2. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



int signalfd(int fd, const sigset_t* mask)
{
    size_t sz;
    sz = sizeof(sigset_t);
    return syscall(__NR_signalfd, fd, mask, sz);
}



struct lwevent_signal {
    sigset_t mask;
    lwevent_signalcb callback;
    struct lwevent* ev;
};



static void _lwevent_signalfd_callback(struct lwevent* ev, int revents)
{
    (void)revents;
    struct signalfd_siginfo ssi;
    struct lwevent_signal* si;

    si = lwevent_get_user(ev);

    while(1) {
        if(TEMP_FAILURE_RETRY( read(lwevent_get_fd(ev), &ssi, sizeof(ssi)) ) == -1) {
            if(errno == EAGAIN) return;
            /* TODO: now what??? */
        }

        si->callback(&ssi);
    }
}

struct lwevent_signal* lwevent_signalfd(const sigset_t* mask, lwevent_signalcb callback)
{
    struct lwevent_signal* si;
    int fd;

    si = malloc(sizeof(struct lwevent_signal));
    if(!si) return 0;
    memset(si, 0, sizeof(struct lwevent_signal));

    si->mask = *mask;
    sigdelset(&si->mask, SIGBUS);
    sigdelset(&si->mask, SIGFPE);
    sigdelset(&si->mask, SIGILL);
    sigdelset(&si->mask, SIGSEGV);
    si->callback = callback;

    fd = signalfd(-1, &si->mask);
    if(fd == -1) {
        free(si);
        return 0;
    }

    /* TODO: make non-blocking */

    si->ev = lwevent_new(fd, EPOLLIN, _lwevent_signalfd_callback);
    if(!si->ev) {
        close(fd);
        free(si);
        return 0;
    }

    lwevent_set_user(si->ev, si, 0);
    return si;
}



void lwevent_signalfd_getsigs(const struct lwevent_signal* si, sigset_t* mask)
{
    *mask = si->mask;
}

    

lwevent_signalcb lwevent_signalfd_getcb(const struct lwevent_signal* si)
{
    return si->callback;
}    



int lwevent_signalfd_setsigs(struct lwevent_signal* si, const sigset_t* mask, sigset_t* old_mask)
{
    sigset_t mnew;

    if(old_mask) *old_mask = si->mask;

    mnew = *mask;
    sigdelset(&mnew, SIGBUS);
    sigdelset(&mnew, SIGFPE);
    sigdelset(&mnew, SIGILL);
    sigdelset(&mnew, SIGSEGV);

    if(signalfd(lwevent_get_fd(si->ev), &mnew)) return -1;
    si->mask = mnew;
    return 0;
}



void lwevent_signalfd_setcb(struct lwevent_signal* si, lwevent_signalcb callback, 
    lwevent_signalcb* old_callback)
{
    if(old_callback) *old_callback = si->callback;
    si->callback = callback;
}



void lwevent_signalfd_close(struct lwevent_signal* si)
{
    if(!si) return;

    lwevent_free(si->ev);
    free(si);
}



struct lwevent_signal* lwevent_signalfd_default(void) 
{
    /* TODO */
    return lwevent_signalfd(...);
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
